from irekua_database.models import LicenceType
from .utils import BaseFilter


search_fields = (
    'name',
)


class Filter(BaseFilter):
    class Meta:
        model = LicenceType
        fields = (
            'name',
            'years_valid_for',
            'can_view',
            'can_download',
            'can_view_annotations',
            'can_annotate',
            'can_vote_annotations'
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import MetaCollection

from irekua_rest_api.serializers.users import users


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = MetaCollection
        fields = (
            'url',
            'name',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = MetaCollection
        fields = (
            'url',
            'name',
            'description',
        )


class DetailSerializer(serializers.ModelSerializer):
    created_by = users.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = MetaCollection
        fields = (
            'url',
            'name',
            'description',
            'created_by',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = MetaCollection
        fields = (
            'name',
            'description',
        )

    def create(self, validated_data):
        user = self.context['request'].user
        validated_data['created_by'] = user
        return super().create(validated_data)


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = MetaCollection
        fields = (
            'description',
        )



from irekua_database.models import CollectionUser
from .utils import BaseFilter


search_fields = (
    'user__username',
    'user__first_name',
    'user__last_name',
)


class Filter(BaseFilter):
    class Meta:
        model = CollectionUser
        fields = (
            'user__username',
            'user__first_name',
            'user__last_name',
            'user__institution__institution_name',
            'user__institution__institution_code',
            'user__institution__subdependency',
            'user__is_superuser',
            'user__is_curator',
            'user__is_model',
            'user__is_developer',
            'role',
        )



from rest_framework.permissions import BasePermission
from irekua_permissions.annotations import (
    annotations as annotation_permissions)


class CanAnnotate(BasePermission):
    def has_permission(self, request, view):
        item = view.get_object()
        user = request.user
        return annotation_permissions.create(user, item=item)


class IsCreator(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        return user == obj.created_by


class HasUpdatePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        sampling_event = obj.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(user, 'change_collection_item')


class HasViewPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        sampling_event = obj.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(user, 'view_collection_item')


class HasViewAnnotationsPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        sampling_event = obj.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(user, 'view_collection_annotations')

class HasDownloadPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        sampling_event = obj.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(user, 'download_collection_items')


class HasAddAnnotationPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        sampling_event = obj.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.has_permission(user, 'add_collection_annotation')


class IsCollectionAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        sampling_event = obj.sampling_event_device.sampling_event
        collection = sampling_event.collection

        return collection.is_admin(user)


class IsCollectionTypeAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        sampling_event = obj.sampling_event_device.sampling_event
        collection = sampling_event.collection
        collection_type = collection.collection_type

        return collection_type.is_admin(user)


class ItemIsOpenToView(BasePermission):
    def has_object_permission(self, request, view, obj):
        licence = obj.licence

        if not licence.is_active:
            return True

        licence_type = licence.licence_type
        return licence_type.can_view


class ItemIsOpenToDownload(BasePermission):
    def has_object_permission(self, request, view, obj):
        licence = obj.licence

        if not licence.is_active:
            return True

        licence_type = licence.licence_type
        return licence_type.can_download


class ItemIsOpenToAnnotate(BasePermission):
    def has_object_permission(self, request, view, obj):
        licence = obj.licence

        if not licence.is_active:
            return True

        licence_type = licence.licence_type
        return licence_type.can_annotate


class ItemIsOpenToViewAnnotations(BasePermission):
    def has_object_permission(self, request, view, obj):
        licence = obj.licence

        if not licence.is_active:
            return True

        licence_type = licence.licence_type
        return licence_type.can_view_annotation



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import CollectionItemType

from irekua_rest_api.serializers.object_types import items

from . import types
from irekua_rest_api.serializers.object_types import events
from irekua_rest_api.serializers.object_types import mime_types


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = CollectionItemType
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='rest-api:collectionitemtype-detail')
    event_types = events.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.event_types')
    mime_types = mime_types.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.mime_types')


    class Meta:
        model = CollectionItemType
        fields = (
            'url',
            'id',
            'item_type',
            'event_types',
            'mime_types',
            'metadata_schema',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    item_type = items.SelectSerializer(
        many=False,
        read_only=True)
    collection_type = types.SelectSerializer(
        many=False,
        read_only=True)
    event_types = events.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.event_types')
    mime_types = mime_types.SelectSerializer(
        many=True,
        read_only=True,
        source='item_type.mime_types')

    class Meta:
        model = CollectionItemType
        fields = (
            'url',
            'id',
            'collection_type',
            'item_type',
            'metadata_schema',
            'event_types',
            'mime_types',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = CollectionItemType
        fields = (
            'item_type',
            'metadata_schema',
        )

    def create(self, validated_data):
        collection_type = self.context['collection_type']
        validated_data['collection_type'] = collection_type
        return super().create(validated_data)



from uuid import uuid4

from rest_framework.test import APITestCase
from irekua_database.utils import simple_JSON_schema
from irekua_database.models import DeviceType, DeviceBrand
from irekua_rest_api.serializers import devices
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class DeviceTestCase(BaseTestCase, APITestCase):
    serializer = devices.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: Users.ALL_AUTHENTICATED_USERS,
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
        ],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
        ],
        Actions.DESTROY: [
            Users.ADMIN,
        ],
    })

    def setUp(self):
        super().setUp()
        self.device_type = DeviceType.objects.create(
            name=str(uuid4()),
            description='random device type')

        self.brand = DeviceBrand.objects.create(
            name=str(uuid4()))

    def generate_random_json_data(self):
        data = {
            'device_type': self.device_type.pk,
            'brand': self.brand.pk,
            'model': str(uuid4()),
            'metadata_schema': simple_JSON_schema(),
            'configuration_schema': simple_JSON_schema(),
        }
        return data



from irekua_database.models import Collection
from django_filters import FilterSet


search_fields = (
    'user__username',
    'user__first_name',
    'user__last_name',
)


class Filter(FilterSet):
    class Meta:
        model = Collection.administrators.through
        fields = (
            'user__username',
            'user__first_name',
            'user__last_name',
        )



from irekua_database.models import PhysicalDevice
from .utils import BaseFilter


search_fields = (
    'device__brand__name',
    'device__model',
    'device__device_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = PhysicalDevice
        fields = (
            'device__brand__name',
            'device__model',
            'device__device_type__name',
            'created_by__username',
            'created_by__first_name'
        )



from irekua_database.models import Entailment
from .utils import BaseFilter


search_fields = (
    'source__value',
    'source__term_type__name',
    'target__value',
    'target__term_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = Entailment
        fields = (
            'source__value',
            'source__term_type__name',
            'target__value',
            'target__term_type__name',
        )



from . import devices
from . import sites
from . import users
from . import data_collections
from . import metacollections
from . import administrators


__all__ = [
    'devices',
    'sites',
    'users',
    'data_collections',
    'metacollections',
    'administrators',
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.models import Permission

from rest_framework import mixins
from rest_framework.decorators import action
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils

from irekua_rest_api.permissions import ReadOnly
from irekua_rest_api.permissions import IsAdmin


class RoleViewSet(mixins.UpdateModelMixin,
                  mixins.DestroyModelMixin,
                  mixins.RetrieveModelMixin,
                  utils.CustomViewSetMixin,
                  GenericViewSet):
    queryset = models.Role.objects.all()  # pylint: disable=E1101

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.users.roles)
        .extend(
            add_permission=serializers.users.roles.SelectPermissionSerializer,
            remove_permission=serializers.users.roles.SelectPermissionSerializer
        ))

    permission_mapping = utils.PermissionMapping(default=IsAdmin | ReadOnly)

    @action(detail=True, methods=['POST'])
    def add_permission(self, request, pk=None):
        return self.add_related_object_view(
            Permission, 'permission')

    @action(detail=True, methods=['POST'])
    def remove_permission(self, request, pk=None):
        return self.remove_related_object_view(
            'permission')



from uuid import uuid4

from rest_framework.test import APITestCase

from irekua_database.models import User, TermType
from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import term_suggestions
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class DummyRequest(object):
    def __init__(self, user):
        self.user = user


class TermSuggestionTestCase(BaseTestCase, APITestCase):
    serializer = term_suggestions.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: Users.ALL_AUTHENTICATED_USERS,
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN],
        Actions.DESTROY: [
            Users.ADMIN],
    })

    def setUp(self):
        super(TermSuggestionTestCase, self).setUp()
        self.random_user = User.objects.create(
            username=str(uuid4()),
            password=str(uuid4()))
        self.term_type = TermType.objects.create(
            name=str(uuid4()),
            description='Random term type',
            is_categorical=True,
            metadata_schema=simple_JSON_schema(),
            synonym_metadata_schema=simple_JSON_schema())

    def get_serializer_context(self):
        request = DummyRequest(user=self.random_user)
        return {'request': request}

    def generate_random_json_data(self):
        data = {
            'term_type': self.term_type.pk,
            'value': str(uuid4()),
            'description': 'Random term suggestion',
            'metadata': {}
        }
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import SecondaryItem

from irekua_rest_api.serializers import object_types
from . import items


class SelectSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = SecondaryItem
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = SecondaryItem
        fields = (
            'url',
            'id',
            'item',
            'item_type'
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    item_type = object_types.items.SelectSerializer(many=False, read_only=True)
    item = items.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = SecondaryItem
        fields = (
            'url',
            'id',
            'hash',
            'item_type',
            'item',
            'media_info',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = SecondaryItem
        fields = (
            'hash',
            'item_file',
            'item_type',
            'item',
            'media_info',
        )

    def create(self, validated_data):
        item = self.context['item']
        validated_data['item'] = item
        return super().create(validated_data)


class DownloadSerializer(serializers.ModelSerializer):
    class Meta:
        model = SecondaryItem
        fields = (
            'item_file',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Term

from irekua_rest_api.serializers.object_types import terms


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        models = Term
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Term
        fields = (
            'url',
            'term_type',
            'value',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    term_type = terms.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = Term
        fields = (
            'url',
            'id',
            'term_type',
            'value',
            'description',
            'metadata',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Term
        fields = (
            'value',
            'description',
            'metadata'
        )

    def create(self, validated_data):
        validated_data['term_type'] = self.context['term_type']
        return super().create(validated_data)


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Term
        fields = (
            'description',
            'metadata'
        )


class EntailmentSerializer(serializers.Serializer):
    term_type = serializers.CharField(
        source='target.term_type')
    description = serializers.CharField(
        source='target.description')
    value = serializers.CharField(
        source='target.value')
    id = serializers.IntegerField(
        source='target.id')
    scope = serializers.CharField(
        source='target.scope')


class TermSerializer(serializers.ModelSerializer):
    term_type = serializers.StringRelatedField(many=False)

    class Meta:
        model = Term
        fields = [
            'id',
            'scope',
            'term_type',
            'value',
            'description',
        ]


class ComplexTermSerializer(serializers.ModelSerializer):
    term_type = serializers.StringRelatedField(many=False)
    entailments = EntailmentSerializer(
        many=True,
        read_only=True,
        source='entailment_source')

    class Meta:
        model = Term
        fields = [
            'id',
            'scope',
            'term_type',
            'value',
            'description',
            'entailments',
        ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Collection
from irekua_database.models import CollectionUser

from irekua_rest_api.serializers.object_types.data_collections import types
from irekua_rest_api.serializers.users import institutions


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = Collection
        fields = (
            'url',
            'name',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Collection
        fields = (
            'url',
            'name',
            'logo',
            'collection_type',
            'description',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    collection_type = types.SelectSerializer(
        many=False,
        read_only=False)
    institution = institutions.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = Collection
        fields = (
            'url',
            'name',
            'collection_type',
            'description',
            'logo',
            'metadata',
            'institution',
            'logo',
            'created_on',
            'modified_on',
        )


class UserData(serializers.ModelSerializer):
    class Meta:
        model = CollectionUser
        fields = (
            'role',
            'metadata'
        )


class CreateSerializer(serializers.ModelSerializer):
    user_data = UserData(many=False, read_only=False)

    class Meta:
        model = Collection
        fields = (
            'name',
            'collection_type',
            'description',
            'logo',
            'metadata',
            'institution',
            'user_data',
        )

    def create(self, validated_data):
        user = self.context['request'].user

        user_data = validated_data.pop('user_data')
        collection = Collection.objects.create(**validated_data)  # pylint: disable=E1101

        user_data['user'] = user
        user_data['collection'] = collection
        CollectionUser.objects.create(**user_data)  # pylint: disable=E1101

        collection.add_administrator(user)

        # Strange loading condition requires this line to be called
        # in order to correctly return parsed data in HTTP response
        self.data

        return collection


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Collection
        fields = (
            'description',
            'logo',
            'metadata',
            'institution',
        )



from rest_framework import routers
from irekua_rest_api import views

object_types_router = routers.DefaultRouter()
object_types_router.register(
    r'mime_types',
    views.MimeTypeViewSet)
object_types_router.register(
    r'annotation_types',
    views.AnnotationTypeViewSet)
object_types_router.register(
    r'collection_types',
    views.CollectionTypeViewSet)
object_types_router.register(
    r'device_types',
    views.DeviceTypeViewSet)
object_types_router.register(
    r'entailment_types',
    views.EntailmentTypeViewSet)
object_types_router.register(
    r'event_types',
    views.EventTypeViewSet)
object_types_router.register(
    r'item_types',
    views.ItemTypeViewSet)
object_types_router.register(
    r'licence_types',
    views.LicenceTypeViewSet)
object_types_router.register(
    r'sampling_event_type_device_types',
    views.SamplingEventTypeDeviceTypeViewSet)
object_types_router.register(
    r'sampling_event_type_site_types',
    views.SamplingEventTypeSiteTypeViewSet)
object_types_router.register(
    r'sampling_event_types',
    views.SamplingEventTypeViewSet)
object_types_router.register(
    r'site_types',
    views.SiteTypeViewSet)
object_types_router.register(
    r'collection_type_site_types',
    views.CollectionTypeSiteTypeViewSet)
object_types_router.register(
    r'collection_type_administrators',
    views.CollectionTypeAdministratorViewSet)
object_types_router.register(
    r'collection_type_annotation_types',
    views.CollectionTypeAnnotationTypeViewSet)
object_types_router.register(
    r'collection_type_licence_types',
    views.CollectionTypeLicenceTypeViewSet)
object_types_router.register(
    r'collection_type_sampling_event_types',
    views.CollectionTypeSamplingEventTypeViewSet)
object_types_router.register(
    r'collection_type_item_types',
    views.CollectionTypeItemTypeViewSet)
object_types_router.register(
    r'collection_type_event_types',
    views.CollectionTypeEventTypeViewSet)
object_types_router.register(
    r'collection_type_device_types',
    views.CollectionTypeDeviceTypeViewSet)
object_types_router.register(
    r'collection_type_roles',
    views.CollectionTypeRoleViewSet)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Item

from irekua_rest_api.serializers.object_types import items
from irekua_rest_api.serializers.object_types import events
from irekua_rest_api.serializers.sampling_events import devices
from irekua_rest_api.serializers import licences
from . import tags


class SelectSerializer(serializers.ModelSerializer):
    id = serializers.PrimaryKeyRelatedField(
        many=False,
        read_only=False,
        queryset=Item.objects.all())  # pylint: disable=E1101

    class Meta:
        model = Item
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Item
        fields = (
            'url',
            'id',
            'item_type',
            'captured_on',
        )


class ListCollectionSerializer(serializers.ModelSerializer):
    collection = serializers.CharField(
        source='sampling_event_device.sampling_event.collection',
        read_only=True,
        label="collection")

    class Meta:
        model = Item
        fields = (
            'url',
            'id',
            'item_type',
            'captured_on',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    item_type = items.SelectSerializer(many=False, read_only=True)
    sampling_event_device = devices.SelectSerializer(many=False, read_only=True)
    licence = licences.SelectSerializer(many=False, read_only=True)
    tags = tags.SelectSerializer(many=True, read_only=True)
    ready_event_types = events.SelectSerializer(many=True, read_only=True)

    class Meta:
        model = Item
        fields = (
            'url',
            'id',
            'hash',
            'item_type',
            'media_info',
            'metadata',
            'sampling_event_device',
            'captured_on',
            'licence',
            'tags',
            'ready_event_types',
            'created_on',
            'modified_on',
            'created_by',
            'modified_by',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Item
        fields = (
            'hash',
            'item_file',
            'item_type',
            'media_info',
            'metadata',
            'captured_on',
            'licence',
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        licence_field = self.fields['licence']
        item_type_field = self.fields['item_type']

        try:
            sampling_event_device = self.context['sampling_event_device']

            collection = sampling_event_device.sampling_event.collection
            licence_field.queryset = collection.licence_set.all()

            collection_type = collection.collection_type
            if collection_type.restrict_item_types:
                item_type_field.queryset = (
                    collection_type.item_types.all()
                )

        except (KeyError, AttributeError):
            pass

    def create(self, validated_data):
        user = self.context['request'].user
        sampling_event_device = self.context['sampling_event_device']

        validated_data['sampling_event_device'] = sampling_event_device
        validated_data['created_by'] = user
        validated_data['modified_by'] = user
        return super().create(validated_data)


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Item
        fields = (
            'metadata',
        )


    def update(self, validated_data):
        user = self.context['request'].user
        validated_data['modified_by'] = user
        return super().update(validated_data)


class DownloadSerializer(serializers.ModelSerializer):
    class Meta:
        model = Item
        fields = (
            'item_file',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import CollectionType
from irekua_database.models import SiteType

from irekua_rest_api.serializers.object_types import sites
from . import types


MODEL = CollectionType.site_types.through  # pylint: disable=E1101


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    site_type = serializers.PrimaryKeyRelatedField(
        many=False,
        read_only=True,
        source='sitetype')

    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
            'site_type',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    site_type = sites.SelectSerializer(
        many=False,
        read_only=True,
        source='sitetype')
    collection_type = types.SelectSerializer(
        many=False,
        read_only=True,
        source='collectiontype')

    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
            'collection_type',
            'site_type',
        )


class CreateSerializer(serializers.ModelSerializer):
    site_type = serializers.PrimaryKeyRelatedField(
        many=False,
        read_only=False,
        queryset=SiteType.objects.all(),  # pylint: disable=E1101
        source='sitetype')

    class Meta:
        model = MODEL
        fields = (
            'site_type',
        )

    def create(self, validated_data):
        collection_type = self.context['collection_type']
        validated_data['collectiontype'] = collection_type
        return super().create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
import irekua_rest_api.permissions.synonym_suggestions as permissions


class SynonymSuggestionViewSet(mixins.UpdateModelMixin,
                               mixins.RetrieveModelMixin,
                               mixins.DestroyModelMixin,
                               utils.CustomViewSetMixin,
                               GenericViewSet):
    queryset = models.SynonymSuggestion.objects.all()  # pylint: disable=E1101

    serializer_mapping = utils.SerializerMapping.from_module(
        serializers.terms.synonym_suggestions)

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated, # TODO: Fix permissions
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            IsAdmin
        ]
    }, default=IsAuthenticated)



from rest_framework import routers
from irekua_rest_api import views


additional_router = routers.DefaultRouter()
additional_router.register(
    r'annotation_votes',
    views.AnnotationVoteViewSet)
additional_router.register(
    r'secondary_items',
    views.SecondaryItemViewSet)
additional_router.register(
    r'annotation_tools',
    views.AnnotationToolViewSet)
additional_router.register(
    r'collection_devices',
    views.CollectionDeviceViewSet)
additional_router.register(
    r'collection_sites',
    views.CollectionSiteViewSet)
additional_router.register(
    r'collection_users',
    views.CollectionUserViewSet)
additional_router.register(
    r'collection_administrators',
    views.CollectionAdministratorViewSet)
additional_router.register(
    r'device_brands',
    views.DeviceBrandViewSet)
additional_router.register(
    r'entailments',
    views.EntailmentViewSet)
additional_router.register(
    r'institutions',
    views.InstitutionViewSet)
additional_router.register(
    r'licences',
    views.LicenceViewSet)
additional_router.register(
    r'metacollections',
    views.MetaCollectionViewSet)
additional_router.register(
    r'physical_devices',
    views.PhysicalDeviceViewSet)
additional_router.register(
    r'roles',
    views.RoleViewSet)
additional_router.register(
    r'synonym_suggestions',
    views.SynonymSuggestionViewSet)
additional_router.register(
    r'synonyms',
    views.SynonymViewSet)
additional_router.register(
    r'tags',
    views.TagViewSet)
additional_router.register(
    r'term_suggestions',
    views.TermSuggestionViewSet)
additional_router.register(
    r'term_instances',
    views.TermViewSet)
additional_router.register(
    r'sampling_event_devices',
    views.SamplingEventDeviceViewSet)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import MimeType


class SelectSerializer(serializers.ModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='rest-api:mimetype-detail')
    class Meta:
        model = MimeType
        fields = (
            'url',
            'mime_type',
            'media_info_schema'
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'url',
            'mime_type',
            'media_info_schema',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'url',
            'mime_type',
            'media_info_schema',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'mime_type',
            'media_info_schema',
        )


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = MimeType
        fields = (
            'mime_type',
            'media_info_schema',
        )



from irekua_database.models import AnnotationTool
from .utils import BaseFilter


search_fields = (
    'name',
)


class Filter(BaseFilter):
    class Meta:
        model = AnnotationTool
        fields = (
            'name',
            'version')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework.decorators import action
from rest_framework.viewsets import ModelViewSet
from rest_framework.response import Response

from irekua_database import models
from irekua_rest_api import filters
from irekua_rest_api import utils
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsDeveloper
from irekua_rest_api.permissions import IsSpecialUser
from irekua_rest_api.permissions import data_collections as permissions


class CollectionViewSet(utils.CustomViewSetMixin, ModelViewSet):
    queryset = models.Collection.objects.all()  # pylint: disable=E1101
    search_fields = filters.data_collections.search_fields
    filterset_class = filters.data_collections.Filter

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.data_collections.data_collections)
        .extend(
            metacollections=serializers.data_collections.metacollections.ListSerializer,
            add_metacollection=serializers.data_collections.metacollections.CreateSerializer,
            licences=serializers.licences.ListSerializer,
            add_licence=serializers.licences.CreateSerializer,
            devices=serializers.data_collections.devices.ListSerializer,
            add_device=serializers.data_collections.devices.CreateSerializer,
            sites=serializers.data_collections.sites.ListSerializer,
            add_site=serializers.data_collections.sites.CreateSerializer,
            users=serializers.data_collections.users.ListSerializer,
            add_user=serializers.data_collections.users.CreateSerializer,
            sampling_events=serializers.sampling_events.sampling_events.ListSerializer,
            add_sampling_event=serializers.sampling_events.sampling_events.CreateSerializer,
            types=serializers.object_types.data_collections.types.ListSerializer,
            add_type=serializers.object_types.data_collections.types.CreateSerializer,
            licence_types=serializers.object_types.licences.ListSerializer,
            add_licence_type=serializers.object_types.licences.CreateSerializer,
            items=serializers.items.items.ListSerializer,
            administrators=serializers.data_collections.administrators.ListSerializer,
            add_administrator=serializers.data_collections.administrators.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.HasUpdatePermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            )
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            permissions.IsCollectionAdmin | IsAdmin
        ],
        'add_licence_type': [
            IsAuthenticated, IsAdmin
        ],
        'add_metacollection': [
            IsAuthenticated, IsAdmin | IsDeveloper
        ],
        'licences': [
            IsAuthenticated,
            (
                permissions.IsCollectionUser |
                IsSpecialUser
            ),
        ],
        'add_licence': [
            IsAuthenticated,
            (
                permissions.HasAddLicencePermission |
                permissions.IsCollectionAdmin
            ),
        ],
        'devices': [
            IsAuthenticated,
            (
                permissions.IsCollectionUser |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            )
        ],
        'add_device': [
            IsAuthenticated,
            (
                permissions.HasAddDevicePermission |
                permissions.IsCollectionAdmin |
                IsAdmin
            )
        ],
        'sites': [
            IsAuthenticated,
            (
                permissions.IsCollectionUser |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            )
        ],
        'add_site': [
            IsAuthenticated,
            (
                permissions.HasAddSitePermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            )
        ],
        'users': [
            IsAuthenticated,
            (
                permissions.IsCollectionUser |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            )
        ],
        'add_user': [
            IsAuthenticated,
            (
                permissions.HasAddUserPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            )
        ],
        'sampling_events': [
            IsAuthenticated,
            (
                permissions.IsCollectionUser |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            )
        ],
        'add_sampling_event': [
            IsAuthenticated,
            (
                permissions.HasAddItemPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            )
        ],
        'items': [
            IsAuthenticated,
            (
                permissions.IsCollectionUser |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            )
        ],
        'add_type': [IsAuthenticated, IsAdmin],
        'add_administrator': [
            IsAuthenticated,
            (
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            )
        ],
    })

    def get_object(self):
        collection_id = self.kwargs['pk']
        collection = get_object_or_404(models.Collection, pk=collection_id)

        self.check_object_permissions(self.request, collection)
        return collection

    def get_serializer_context(self):
        context = super().get_serializer_context()
        try:
            collection = self.get_object()
        except (KeyError, AssertionError, AttributeError):
            collection = None

        context['collection'] = collection
        return context

    def get_queryset(self):
        if self.action == 'metacollections':
            return models.MetaCollection.objects.all()  # pylint: disable=E1101

        if self.action == 'licences':
            collection_id = self.kwargs['pk']
            return models.Licence.objects.filter(collection=collection_id)  # pylint: disable=E1101

        if self.action == 'devices':
            collection_id = self.kwargs['pk']
            return models.CollectionDevice.objects.filter(collection=collection_id)  # pylint: disable=E1101

        if self.action == 'sites':
            collection_id = self.kwargs['pk']
            return models.CollectionSite.objects.filter(collection=collection_id)  # pylint: disable=E1101

        if self.action == 'users':
            collection_id = self.kwargs['pk']
            return models.CollectionUser.objects.filter(collection=collection_id)  # pylint: disable=E1101

        if self.action == 'sampling_events':
            collection_id = self.kwargs['pk']
            return models.SamplingEvent.objects.filter(collection=collection_id)  # pylint: disable=E1101

        if self.action == 'types':
            return models.CollectionType.objects.all()  # pylint: disable=E1101

        if self.action == 'licence_types':
            return models.LicenceType.objects.all()  # pylint: disable=E1101

        if self.action == 'items':
            collection_id = self.kwargs['pk']
            return models.Item.objects.filter(  # pylint: disable=E1101
                sampling_event_device__sampling_event__collection=collection_id)

        if self.action == 'administrators':
            model = models.Collection.administrators.through
            collection_id = self.kwargs['pk']
            return model.objects.filter(collection=collection_id)

        if self.action == 'item_locations':
            collection_id = self.kwargs['pk']
            return self.get_item_locations_queryset(collection_id)

        if self.action == 'site_locations':
            collection_id = self.kwargs['pk']
            return self.get_site_locations_queryset(collection_id)

        if self.action == 'sampling_event_locations':
            collection_id = self.kwargs['pk']
            return self.get_sampling_event_locations_queryset(collection_id)

        return super().get_queryset()

    def get_item_locations_queryset(self, collection_id):
        # TODO: Fix queryset filtering for user
        return models.Item.objects.filter(  # pylint: disable=E1101
            sampling_event_device__sampling_event__collection=collection_id)

    def get_site_locations_queryset(self, collection_id):
        # TODO: Fix queryset filtering for user
        return models.CollectionSite.objects.filter(collection=collection_id) # pylint: disable=E1101

    def get_sampling_event_locations_queryset(self, collection_id):
        # TODO: Fix queryset filtering for user
        return models.SamplingEvent.objects.filter(collection=collection_id) # pylint: disable=E1101

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.metacollections.Filter,
        search_fields=filters.metacollections.search_fields)
    def metacollections(self, request):
        return self.list_related_object_view()

    @metacollections.mapping.post
    def add_metacollection(self, request):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.licences.Filter,
        search_fields=filters.licences.search_fields)
    def licences(self, request, pk=None):
        return self.list_related_object_view()

    @licences.mapping.post
    def add_licence(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.collection_devices.Filter,
        search_fields=filters.collection_devices.search_fields)
    def devices(self, request, pk=None):
        return self.list_related_object_view()

    @devices.mapping.post
    def add_device(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.collection_sites.Filter,
        search_fields=filters.collection_sites.search_fields)
    def sites(self, request, pk=None):
        return self.list_related_object_view()

    @sites.mapping.post
    def add_site(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.collection_users.Filter,
        search_fields=filters.collection_users.search_fields)
    def users(self, request, pk=None):
        return self.list_related_object_view()

    @users.mapping.post
    def add_user(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.sampling_events.Filter,
        search_fields=filters.sampling_events.search_fields)
    def sampling_events(self, request, pk=None):
        return self.list_related_object_view()

    @sampling_events.mapping.post
    def add_sampling_event(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.collection_types.Filter,
        search_fields=filters.collection_types.search_fields)
    def types(self, request):
        return self.list_related_object_view()

    @types.mapping.post
    def add_type(self, request):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.licence_types.Filter,
        search_fields=filters.licence_types.search_fields)
    def licence_types(self, request):
        return self.list_related_object_view()

    @licence_types.mapping.post
    def add_licence_type(self, request):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.collection_administrators.Filter,
        search_fields=filters.collection_administrators.search_fields)
    def administrators(self, request, pk=None):
        return self.list_related_object_view()

    @administrators.mapping.post
    def add_administrator(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def items(self, request, pk=None):
        return self.list_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.collection_sites.Filter,
        search_fields=filters.collection_sites.search_fields)
    def site_locations(self, request, pk=None):
        queryset = self.filter_queryset(self.get_queryset())

        serializer = serializers.sites.CollectionSiteLocationSerializer(
            queryset,
            many=True,
            read_only=True)
        return Response(serializer.data)

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.items.Filter,
        search_fields=filters.items.search_fields)
    def item_locations(self, request, pk=None):
        queryset = self.filter_queryset(self.get_queryset())

        serializer = serializers.sites.ItemLocationSerializer(
            queryset,
            many=True,
            read_only=True)
        return Response(serializer.data)

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.sampling_events.Filter,
        search_fields=filters.sampling_events.search_fields)
    def sampling_event_locations(self, request, pk=None):
        queryset = self.filter_queryset(self.get_queryset())

        serializer = serializers.sites.SamplingEventLocationSerializer(
            queryset,
            many=True,
            read_only=True)
        return Response(serializer.data)



from django.conf.urls import url, include
from rest_framework.documentation import include_docs_urls


urlpatterns = [
    url('v1/', include(('irekua_rest_api.urls.api', 'rest_api'), namespace='v1')),
    url('docs/', include_docs_urls(title='Irekua REST API documentation')),
    url('auth/', include('rest_framework.urls', namespace='rest_framework')),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import AnnotationVote

from irekua_rest_api.serializers.users import users
from . import annotations


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = AnnotationVote
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = AnnotationVote
        fields = (
            'url',
            'id',
            'label',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    annotation = annotations.SelectSerializer(
        many=False,
        read_only=True)
    created_by = users.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = AnnotationVote
        fields = (
            'url',
            'id',
            'annotation',
            'label',
            'created_by',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    label = serializers.JSONField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        try:
            annotation = self.context['annotation']
            previous_label = annotation.label

            self.fields['label'].initial = previous_label
        except (KeyError, AttributeError):
            pass

    class Meta:
        model = AnnotationVote
        fields = (
            'label',
        )

    def create(self, validated_data):
        annotation = self.context['annotation']
        user = self.context['request'].user

        validated_data['annotation'] = annotation
        validated_data['created_by'] = user
        return super().create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Device

from irekua_rest_api.serializers.object_types import devices
from . import brands


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = Device
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Device
        fields = (
            'url',
            'id',
            'device_type',
            'brand',
            'model',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    device_type = devices.SelectSerializer(
        many=False,
        read_only=True)
    brand = brands.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = Device
        fields = (
            'url',
            'id',
            'device_type',
            'brand',
            'model',
            'metadata_schema',
            'configuration_schema',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Device
        fields = (
            'device_type',
            'brand',
            'model',
            'metadata_schema',
            'configuration_schema',
        )



from rest_framework.permissions import BasePermission


class IsCreator(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        creator = obj.created_by
        return user == creator


class HasChangePermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        return collection.has_permission(
            user, 'change_collection_sampling_events')


class HasViewPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        return collection.has_permission(
            user, 'view_collection_sampling_events')


class IsCollectionAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        return collection.is_admin(user)


class IsCollectionTypeAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        collection_type = collection.collection_type
        return collection_type.is_admin(user)


class HasViewItemsPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        return collection.has_permission(
            user, 'view_collection_item')



import django_filters

from irekua_database.models import CollectionSite
from .utils import BaseFilter


search_fields = (
    'site__name',
    'site_type',
    'site__locality',
)


class Filter(BaseFilter):
    latitude__gt = django_filters.NumberFilter(
        field_name='site__latitude',
        lookup_expr='gt')
    latitude__lt = django_filters.NumberFilter(
        field_name='site__latitude',
        lookup_expr='lt')

    longitude__gt = django_filters.NumberFilter(
        field_name='site__longitude',
        lookup_expr='gt')
    longitude__lt = django_filters.NumberFilter(
        field_name='site__longitude',
        lookup_expr='lt')

    altitude__gt = django_filters.NumberFilter(
        field_name='site__altitude',
        lookup_expr='gt')
    altitude__lt = django_filters.NumberFilter(
        field_name='site__altitude',
        lookup_expr='lt')

    class Meta:
        model = CollectionSite
        fields = (
            'site__name',
            'site_type',
            'site__locality',
            'site__created_by',
            'created_by',
            'internal_id',
        )



from irekua_database.models import ItemType
from .utils import BaseFilter


search_fields = (
    'name',
)


class Filter(BaseFilter):
    class Meta:
        model = ItemType
        fields = (
            'name',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.response import Response
from rest_framework import status
from rest_framework.viewsets import GenericViewSet
from rest_framework.decorators import action

from irekua_database import models
from irekua_rest_api import utils
from irekua_rest_api import serializers

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsSpecialUser
from irekua_rest_api.permissions import collection_users as permissions


class CollectionUserViewSet(mixins.UpdateModelMixin,
                            mixins.RetrieveModelMixin,
                            mixins.DestroyModelMixin,
                            utils.CustomViewSetMixin,
                            GenericViewSet):
    queryset = models.CollectionUser.objects.all()  # pylint: disable=E1101

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(
            serializers.data_collections.users)
        .extend(
            change_role=serializers.data_collections.users.RoleSerializer
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.IsSelf |
                permissions.HasUpdatePermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            ),
        ],
        utils.Actions.RETRIEVE: [
            IsAuthenticated,
            (
                permissions.IsSelf |
                permissions.IsInCollection |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            ),
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            (
                permissions.IsSelf |
                IsAdmin
            ),
        ],
        'change_role': [
            (
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsAdmin
            )
        ],
    })

    @action(detail=True, methods=['POST'])
    def change_role(self, request, pk=None):
        instance = self.get_object()
        serializer = self.get_serializer(
            instance,
            data=request.data,
            partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import SamplingEventDevice

from irekua_rest_api.serializers.data_collections import devices
from irekua_rest_api.serializers import licences
from irekua_rest_api.serializers.users import users
from . import sampling_events


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = SamplingEventDevice
        fields = (
            'url',
            'id'
        )


class ListSerializer(serializers.ModelSerializer):
    device_type = serializers.CharField(
        read_only=True,
        source='collection_device.physical_device.device.device_type.name')
    device_internal_id = serializers.CharField(
        read_only=True,
        source='collection_device.internal_id')

    class Meta:
        model = SamplingEventDevice
        fields = (
            'url',
            'id',
            'sampling_event',
            'device_type',
            'device_internal_id',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    collection_device = devices.SelectSerializer(
        many=False,
        read_only=True)
    sampling_event = sampling_events.SelectSerializer(
        many=False,
        read_only=True)
    licence = licences.SelectSerializer(
        many=False,
        read_only=True)
    created_by = users.SelectSerializer(
        many=False,
        read_only=True)
    modified_by = users.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = SamplingEventDevice
        fields = (
            'url',
            'id',
            'sampling_event',
            'collection_device',
            'commentaries',
            'metadata',
            'configuration',
            'licence',
            'created_by',
            'created_on',
            'modified_on',
            'modified_by',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = SamplingEventDevice
        fields = (
            'collection_device',
            'commentaries',
            'metadata',
            'configuration',
            'licence',
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        try:
            sampling_event = self.context['sampling_event']
            collection = sampling_event.collection

            collection_device = self.fields['collection_device']
            collection_device.queryset = collection.collectiondevice_set.all()

            licences_field = self.fields['licence']
            licences_field.queryset = collection.licence_set.all()

        except (KeyError, AttributeError):
            pass

    def create(self, validated_data):
        sampling_event = self.context['sampling_event']
        user = self.context['request'].user

        validated_data['sampling_event'] = sampling_event
        validated_data['created_by'] = user
        validated_data['modified_by'] = user
        return super().create(validated_data)


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = SamplingEventDevice
        fields = (
            'commentaries',
            'metadata',
            'configuration',
        )

    def update(self, validated_data):
        user = self.context['request'].user
        validated_data['modified_by'] = user
        return super().create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework import mixins
from rest_framework.decorators import action
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils


class SamplingEventTypeViewSet(mixins.RetrieveModelMixin,
                               mixins.DestroyModelMixin,
                               utils.CustomViewSetMixin,
                               GenericViewSet):
    queryset = models.SamplingEventType.objects.all()  # pylint: disable=E1101

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.object_types.sampling_events.types)
        .extend(
            device_types=serializers.object_types.sampling_events.devices.ListSerializer,
            add_device_types=serializers.object_types.sampling_events.devices.CreateSerializer,
            site_types=serializers.object_types.sampling_events.sites.ListSerializer,
            add_site_types=serializers.object_types.sampling_events.sites.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping()

    def get_object(self):
        sampling_event_pk = self.kwargs['pk']
        sampling_event = get_object_or_404(
            models.SamplingEventType,
            pk=sampling_event_pk)

        self.check_object_permissions(self.request, sampling_event)
        return sampling_event

    def get_serializer_context(self):
        context = super().get_serializer_context()

        try:
            sampling_event_type = self.get_object()
        except (KeyError, AttributeError, AssertionError):
            sampling_event_type = None

        context['sampling_event_type'] = sampling_event_type
        return context

    def get_queryset(self):
        if self.action == 'device_types':
            sampling_event_type_id = self.kwargs['pk']
            queryset = (
                models.SamplingEventTypeDeviceTypeViewSet  # pylint: disable=E1101
                .objects
                .filter(sampling_event_type=sampling_event_type_id))
            return queryset

        if self.action == 'site_types':
            model = models.SamplingEventType.site_types.through  # pylint: disable=E1101
            sampling_event_type_id = self.kwargs['pk']
            return model.objects.filter(
                samplingeventtype_id=sampling_event_type_id)

        return super().get_queryset()

    @action(detail=True, methods=['GET'])
    def device_types(self, request, pk=None):
        return self.list_related_object_view()

    @device_types.mapping.post
    def add_device_types(self, request, pk=None):
        return self.create_related_object_view()

    @action(detail=True, methods=['GET'])
    def site_types(self, request, pk=None):
        return self.list_related_object_view()

    @site_types.mapping.post
    def add_site_types(self, request, pk=None):
        return self.create_related_object_view()



from uuid import uuid4
from random import choice

from rest_framework.test import APITestCase
from irekua_database.utils import simple_JSON_schema
from irekua_database.models import TermType, EntailmentType, Term
from irekua_rest_api.serializers import entailments
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class EntailmentTestCase(BaseTestCase, APITestCase):
    serializer = entailments.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN,
            Users.CURATOR],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            Users.CURATOR],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            Users.CURATOR],
        Actions.DESTROY: [
            Users.ADMIN,
            Users.CURATOR],
        })

    def setUp(self):
        super().setUp()

        term_type_1 = TermType.objects.create(
            name=str(uuid4()),
            description='Random term type',
            is_categorical=True,
            metadata_schema=simple_JSON_schema(),
            synonym_metadata_schema=simple_JSON_schema())
        term_type_2 = TermType.objects.create(
            name=str(uuid4()),
            description='Random term type',
            is_categorical=True,
            metadata_schema=simple_JSON_schema(),
            synonym_metadata_schema=simple_JSON_schema())

        EntailmentType.objects.create(
            source_type=term_type_1,
            target_type=term_type_2,
            metadata_schema=simple_JSON_schema())

        self.type_1_terms = [
            Term.objects.create(
                term_type=term_type_1,
                value=str(uuid4()),
                description='Random term',
                metadata={})
            for _ in range(40)]

        self.type_2_terms = [
            Term.objects.create(
                term_type=term_type_2,
                value=str(uuid4()),
                description='Random term',
                metadata={})
            for _ in range(40)]

    def generate_random_json_data(self):
        term_1 = choice(self.type_1_terms)
        term_2 = choice(self.type_2_terms)
        data = {
            'source': term_1.pk,
            'target': term_2.pk,
            'description': 'Random entailment',
            'metadata': {}
        }
        return data



from rest_framework.permissions import BasePermission


class IsFromInstitution(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        return obj.has_user(user)



from irekua_database.models import Collection
from .utils import BaseFilter


search_fields = (
    'name',
    'collection_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = Collection
        fields = (
            'name',
            'collection_type__name',
            'institution__institution_code',
            'institution__institution_name',
            'institution__country',
        )



import django_filters

from irekua_database.models import Item
from .utils import BaseFilter


search_fields = (
    'item_type__name',
)


class Filter(BaseFilter):
    is_uploaded = django_filters.BooleanFilter(
        field_name='item_file',
        method='is_uploaded_filter',
        label='is uploaded')

    def is_uploaded_filter(self, queryset, name, value):
        return queryset.filter(item_file__isnull=False)

    class Meta:
        model = Item
        fields = (
            'is_uploaded',
            'item_type',
            'sampling_event_device__sampling_event__sampling_event_type',
            'sampling_event_device__sampling_event__collection',
            'sampling_event_device__sampling_event__collection__collection_type',
            'sampling_event_device__sampling_event__collection_site__site_type',
            'sampling_event_device__collection_device__physical_device__device__device_type',
            'sampling_event_device__collection_device__physical_device__device__brand',
        )



from . import administrators
from . import annotations
from . import devices
from . import events
from . import items
from . import licences
from . import roles
from . import sampling_events
from . import sites
from . import types


__all__ = [
    'administrators',
    'annotations',
    'devices',
    'events',
    'items',
    'licences',
    'roles',
    'sampling_events',
    'sites',
    'types',
]



from .actions import Actions
from .actions import AdditionalActionsMixin
from .serializers import SerializerMapping
from .serializers import SerializerMappingMixin
from .permissions import PermissionMapping
from .permissions import PermissionMappingMixin


class CustomViewSetMixin(SerializerMappingMixin,
                         PermissionMappingMixin,
                         AdditionalActionsMixin):
    pass


__all__ = [
    'SerializerMapping',
    'PermissionMapping',
    'CustomViewSetMixin',
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework.viewsets import ModelViewSet
from rest_framework.decorators import action

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import filters
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsAuthenticated



class TermTypeViewSet(utils.CustomViewSetMixin, ModelViewSet):
    queryset = models.TermType.objects.all()  # pylint: disable=E1101
    search_fields = filters.term_types.search_fields
    filterset_class = filters.term_types.Filter

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.object_types.terms)
        .extend(
            terms=serializers.terms.terms.ListSerializer,
            add_term=serializers.terms.terms.CreateSerializer,
            suggestions=serializers.terms.suggestions.ListSerializer,
            suggest_term=serializers.terms.suggestions.CreateSerializer,
            entailment_types=serializers.object_types.entailments.ListSerializer,
            add_entailment_type=serializers.object_types.entailments.CreateSerializer,
            entailments=serializers.terms.entailments.ListSerializer,
            add_entailment=serializers.terms.entailments.CreateSerializer,
            synonyms=serializers.terms.synonyms.ListSerializer,
            add_synonym=serializers.terms.synonyms.CreateSerializer,
            synonym_suggestions=serializers.terms.synonym_suggestions.ListSerializer,
            suggest_synonym=serializers.terms.synonym_suggestions.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.DESTROY: [IsAuthenticated, IsAdmin],
        utils.Actions.CREATE: [IsAuthenticated, IsAdmin],
        utils.Actions.UPDATE: [IsAuthenticated, IsAdmin],
        'add_term': [IsAuthenticated, IsAdmin],
        'add_entailment_type': [IsAuthenticated, IsAdmin],
        'add_entailment': [IsAuthenticated, IsAdmin],
        'add_synonym': [IsAuthenticated, IsAdmin],
    }, default=IsAuthenticated)

    def get_object(self):
        type_pk = self.kwargs['pk']
        term_type = get_object_or_404(models.TermType, pk=type_pk)

        self.check_object_permissions(self.request, term_type)
        return term_type

    def get_serializer_context(self):
        context = super().get_serializer_context()

        try:
            term_type = self.get_object()
        except (KeyError, AssertionError, AttributeError):
            term_type = None

        context['term_type'] = term_type
        return context

    def get_queryset(self):
        if self.action == 'entailments':
            return models.Entailment.objects.all()  # pylint: disable=E1101

        if self.action == 'entailment_types':
            return models.EntailmentType.objects.all()  # pylint: disable=E1101

        if self.action == 'terms':
            term_type_id = self.kwargs['pk']
            return models.Term.objects.filter(term_type=term_type_id)  # pylint: disable=E1101

        if self.action == 'synonyms':
            term_type_id = self.kwargs['pk']
            return models.Synonym.objects.filter(source__term_type=term_type_id)  # pylint: disable=E1101

        if self.action == 'suggestions':
            term_type_id = self.kwargs['pk']
            return models.TermSuggestion.objects.filter(term_type=term_type_id)  # pylint: disable=E1101

        if self.action == 'synonym_suggestions':
            term_type_id = self.kwargs['pk']
            return models.SynonymSuggestion.objects.filter(source__term_type=term_type_id)  # pylint: disable=E1101

        return super().get_queryset()

    @action(detail=False,
            methods=['GET'],
            filterset_class=filters.entailments.Filter,
            search_fields=filters.entailments.search_fields)
    def entailments(self, request):
        return self.list_related_object_view()

    @entailments.mapping.post
    def add_entailment(self, request):
        self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.entailment_types.Filter,
        search_fields=filters.entailment_types.search_fields)
    def entailment_types(self, request):
        return self.list_related_object_view()

    @entailment_types.mapping.post
    def add_entailment_type(self, request):
        self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.terms.Filter,
        search_fields=filters.terms.search_fields)
    def terms(self, request, pk=None):
        return self.list_related_object_view()

    @terms.mapping.post
    def add_term(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.synonyms.Filter,
        search_fields=filters.synonyms.search_fields)
    def synonyms(self, request, pk=None):
        return self.list_related_object_view()

    @synonyms.mapping.post
    def add_synonym(self, request, pk=None):
        self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.term_suggestions.Filter,
        search_fields=filters.term_suggestions.search_fields)
    def suggestions(self, request, pk=None):
        return self.list_related_object_view()

    @suggestions.mapping.post
    def suggest_term(self, request, pk=None):
        self.create_related_object_view()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.synonym_suggestions.Filter,
        search_fields=filters.synonym_suggestions.search_fields)
    def synonym_suggestions(self, request, pk=None):
        return self.list_related_object_view()

    @synonym_suggestions.mapping.post
    def suggest_synonym(self, request, pk=None):
        self.create_related_object_view()



from rest_framework.permissions import BasePermission


class IsOwner(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user

        device = obj.physical_device
        owner = device.owner

        return user == owner


class HasUpdatePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection

        return collection.has_permission(
            user, 'change_collection_devices')


class HasViewPermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection

        return collection.has_permission(
            user, 'view_collection_devices')


class IsCollectionAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection

        return collection.is_admin(user)


class IsCollectionTypeAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        collection_type = collection.collection_type

        return collection_type.is_admin(user)



from uuid import uuid4

from django.urls import reverse
from rest_framework.test import APITestCase

from irekua_rest_api.serializers import institutions
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class InstitutionTestCase(BaseTestCase, APITestCase):
    serializer = institutions.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: Users.ALL_AUTHENTICATED_USERS,
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
        ],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
        ],
        Actions.DESTROY: [
            Users.ADMIN,
        ],
    })

    @staticmethod
    def generate_random_json_data():
        data = {
            'institution_name': str(uuid4()),
            'institution_code': str(uuid4()),
            'subdependency': str(uuid4())
        }
        return data

    def test_update_own_institution(self):
        institution = self.create_random_object()
        other_institution = self.create_random_object()

        new_data = self.generate_random_json_data()
        new_data['country'] = 'MX'

        self.regular_user.institution = other_institution
        self.regular_user.save()

        self.client.force_authenticate(user=self.regular_user)

        url_name = self.get_url_name(Actions.UPDATE)
        url = reverse(url_name, args=[institution.pk])
        response = self.client.put(url, new_data, format='json')
        self.check_response(
            Actions.UPDATE,
            response,
            False,
            'User not from institution')

        url_name = self.get_url_name(Actions.PARTIAL_UPDATE)
        url = reverse(url_name, args=[institution.pk])
        response = self.client.patch(url, new_data, format='json')
        self.check_response(
            Actions.PARTIAL_UPDATE,
            response,
            False,
            'User not from institution')

        url_name = self.get_url_name(Actions.UPDATE)
        url = reverse(url_name, args=[other_institution.pk])
        response = self.client.put(url, new_data, format='json')
        self.check_response(
            Actions.UPDATE,
            response,
            True,
            'User from institution')

        url_name = self.get_url_name(Actions.PARTIAL_UPDATE)
        url = reverse(url_name, args=[other_institution.pk])
        response = self.client.patch(url, new_data, format='json')
        self.check_response(
            Actions.PARTIAL_UPDATE,
            response,
            True,
            'User from institution')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import TermSuggestion

from irekua_rest_api.serializers.object_types import terms
from irekua_rest_api.serializers.users import users


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = TermSuggestion
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = TermSuggestion
        fields = (
            'url',
            'id',
            'term_type',
            'value',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    term_type = terms.SelectSerializer(many=False, read_only=True)
    suggested_by = users.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = TermSuggestion
        fields = (
            'url',
            'term_type',
            'value',
            'description',
            'metadata',
            'suggested_by',
            'suggested_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = TermSuggestion
        fields = (
            'term_type',
            'value',
            'description',
            'metadata',
        )

    def create(self, validated_data):
        user = self.context['request'].user
        validated_data['suggested_by'] = user
        return super().create(validated_data)


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = TermSuggestion
        fields = (
            'description',
            'metadata',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Institution


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = Institution
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Institution
        fields = (
            'url',
            'id',
            'institution_name',
            'institution_code',
            'subdependency',
            'logo',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Institution
        fields = (
            'url',
            'id',
            'institution_name',
            'institution_code',
            'subdependency',
            'country',
            'postal_code',
            'address',
            'website',
            'logo',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Institution
        fields = (
            'institution_name',
            'institution_code',
            'subdependency',
            'country',
            'postal_code',
            'address',
            'website',
            'logo',
        )



from rest_framework.permissions import IsAuthenticated

from .generic import IsUnauthenticated
from .generic import IsDeveloper
from .generic import IsModel
from .generic import IsCurator
from .generic import IsAdmin
from .generic import ReadOnly
from .generic import IsSpecialUser


__all__ = [
    'IsUnauthenticated',
    'IsDeveloper',
    'IsModel',
    'IsCurator',
    'IsAdmin',
    'IsSpecialUser',
    'IsAuthenticated',
    'ReadOnly',
]



from irekua_database.models import Device
from .utils import BaseFilter


search_fields = (
    'brand__name',
    'model',
    'device_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = Device
        fields = (
            'brand__name',
            'model',
            'device_type__name',
        )



from django.conf.urls import url, include

from .additional import additional_router
from .main import main_router
from .object_types import object_types_router



urlpatterns = [
    url(r'^', include(main_router.urls)),
    url(r'^types/', include(object_types_router.urls)),
    url(r'^additional/', include(additional_router.urls)),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import DeviceBrand


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = DeviceBrand
        fields = (
            'url',
            'name',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = DeviceBrand
        fields = (
            'url',
            'name',
            'logo',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = DeviceBrand
        fields = (
            'url',
            'name',
            'website',
            'logo',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = DeviceBrand
        fields = (
            'name',
            'website',
            'logo',
        )



from uuid import uuid4

from rest_framework.test import APITestCase

from irekua_database.models import User
from irekua_rest_api.serializers import metacollections
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class DummyRequest(object):
    def __init__(self, user):
        self.user = user


class MetaCollectionTestCase(BaseTestCase, APITestCase):
    serializer = metacollections.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN,
            Users.DEVELOPER,
            Users.MODEL,
        ],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER,
            Users.MODEL,
        ],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            Users.DEVELOPER,
            Users.MODEL,
        ],
        Actions.DESTROY: [
            Users.ADMIN,
            Users.DEVELOPER,
            Users.MODEL,
        ],
    })

    def setUp(self):
        super(MetaCollectionTestCase, self).setUp()
        self.random_user = User.objects.create(
            username=str(uuid4()),
            password=str(uuid4()))

    def get_serializer_context(self):
        request = DummyRequest(self.random_user)
        return {'request': request}

    @staticmethod
    def generate_random_json_data():
        data = {
            'name': str(uuid4()),
            'description': 'Random Role',
        }
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import EventType

from . import terms


class SelectSerializer(serializers.ModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='rest-api:eventtype-detail')
    name = serializers.PrimaryKeyRelatedField(
        many=False,
        read_only=False,
        queryset=EventType.objects.all())

    class Meta:
        model = EventType
        fields = (
            'url',
            'name',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = EventType
        fields = (
            'url',
            'name',
            'description',
            'icon',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    term_types = terms.SelectSerializer(
        many=True,
        read_only=True)

    class Meta:
        model = EventType
        fields = (
            'url',
            'name',
            'description',
            'icon',
            'term_types',
            'created_on',
            'modified_on'
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = EventType
        fields = (
            'name',
            'description',
            'icon',
        )


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = EventType
        fields = (
            'description',
            'icon',
        )



from rest_framework.permissions import BasePermission


class IsSelf(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection_user = obj.user
        return user == collection_user


class HasUpdatePermission(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection

        return collection.has_permission(
            user, 'change_collection_users')


class IsCollectionAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        return collection.is_admin(user)


class IsCollectionTypeAdmin(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        collection_type = collection.collection_type

        return collection_type.is_admin(user)


class IsInCollection(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        collection = obj.collection
        return collection.has_user(user)



from rest_framework.permissions import BasePermission


class IsOwnSuggestion(BasePermission):
    def has_object_permission(self, request, view, obj):
        user = request.user
        return obj.suggested_by == user



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import ItemType

from . import events
from . import mime_types


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'url',
            'name',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'url',
            'name',
            'description',
            'icon',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    event_types = events.DetailSerializer(
        many=True,
        read_only=True)
    mime_types = mime_types.SelectSerializer(
        many=True,
        read_only=True)

    class Meta:
        model = ItemType
        fields = (
            'url',
            'name',
            'description',
            'mime_types',
            'icon',
            'event_types',
            'created_on',
            'modified_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'name',
            'description',
            'icon',
        )


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = ItemType
        fields = (
            'description',
            'icon',
        )



import django_filters

from irekua_database.models import SynonymSuggestion


search_fields = (
    'source__value',
    'synonym',
)


class Filter(django_filters.FilterSet):
    created_on__gt = django_filters.DateTimeFilter(
        field_name='created_on',
        lookup_expr='gt')
    created_on__lt = django_filters.DateTimeFilter(
        field_name='created_on',
        lookup_expr='gt')

    class Meta:
        model = SynonymSuggestion
        fields = (
            'source__value',
            'synonym',
            'created_by__username',
            'created_by__first_name',
        )



from irekua_database.models import Annotation
from .utils import BaseFilter


search_fields = (
    'annotation_type__name',
    'event_type__name',
    'created_by__username',
)


class Filter(BaseFilter):
    class Meta:
        model = Annotation
        fields = (
            'annotation_type__name',
            'annotation_tool__name',
            'event_type__name',
            'quality',
            'created_by__username',
            'modified_by__username',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.shortcuts import get_object_or_404
from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet
from rest_framework.decorators import action

from irekua_database import models
from irekua_rest_api import utils
from irekua_rest_api import serializers
from irekua_rest_api import filters

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsCurator
from irekua_rest_api.permissions import IsSpecialUser
from irekua_rest_api.permissions import annotations as permissions


class AnnotationViewSet(mixins.UpdateModelMixin,
                        mixins.RetrieveModelMixin,
                        mixins.DestroyModelMixin,
                        mixins.ListModelMixin,
                        utils.CustomViewSetMixin,
                        GenericViewSet):
    queryset = models.Annotation.objects.all()  # pylint: disable=E1101
    filterset_class = filters.annotations.Filter
    search_fields = filters.annotations.search_fields

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.annotations.annotations)
        .extend(
            vote=serializers.annotations.votes.CreateSerializer,
            votes=serializers.annotations.votes.ListSerializer,
            types=serializers.object_types.annotations.ListSerializer,
            add_type=serializers.object_types.annotations.CreateSerializer,
            tools=serializers.annotations.tools.ListSerializer,
            add_tool=serializers.annotations.tools.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasUpdatePermission |
                IsCurator |
                IsAdmin
            ),
        ],
        utils.Actions.RETRIEVE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            ),
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            permissions.IsCreator | IsAdmin,
        ],
        'vote': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasVotePermission |
                IsCurator |
                IsAdmin
            ),
        ],
        'votes': [
            IsAuthenticated,
            (
                permissions.IsCreator |
                permissions.HasViewPermission |
                permissions.IsCollectionAdmin |
                permissions.IsCollectionTypeAdmin |
                IsSpecialUser
            ),
        ],
        'add_type': [IsAuthenticated, IsAdmin],
    }, default=IsAuthenticated)

    def get_object(self):
        annotation_id = self.kwargs['pk']
        annotation = get_object_or_404(models.Annotation, pk=annotation_id)

        self.check_object_permissions(self.request, annotation)
        return annotation

    def get_serializer_context(self):
        context = super().get_serializer_context()

        try:
            annotation = self.get_object()
        except (KeyError, AssertionError, AttributeError):
            annotation = None

        context['annotation'] = annotation
        return context

    def get_queryset(self):
        if self.action == 'votes':
            annotation_id = self.kwargs['pk']
            return models.AnnotationVote.objects.filter(
                annotation=annotation_id)

        if self.action == 'types':
            return models.AnnotationType.objects.all()

        if self.action == 'tools':
            return models.AnnotationTool.objects.all()

        if self.action == utils.Actions.LIST:
            return self.get_list_queryset()

        return super().get_queryset()

    @action(
        detail=True,
        methods=['GET'],
        filterset_class=filters.annotation_votes.Filter,
        search_fields=filters.annotation_votes.search_fields)
    def votes(self, request, pk=None):
        return self.list_related_object_view()

    @votes.mapping.post
    def vote(self, request, pk=None):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.annotation_types.Filter,
        search_fields=filters.annotation_types.search_fields)
    def types(self, request):
        return self.list_related_object_view()

    @types.mapping.post
    def add_type(self, request):
        return self.create_related_object_view()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.annotation_tools.Filter,
        search_fields=filters.annotation_tools.search_fields)
    def tools(self, request):
        return self.list_related_object_view()

    @tools.mapping.post
    def add_tool(self, request):
        return self.create_related_object_view()

    def get_list_queryset(self):
        user = self.request.user

        if user.is_special:
            return models.Annotation.objects.all()

        collection_type_queryset = self.get_collection_type_admin_queryset(user)
        collection_admin_queryset = self.get_collection_admin_queryset(user)
        view_pemission_queryset = self.get_view_permission_queryset(user)
        open_queryset = self.get_open_queryset()

        return open_queryset.union(collection_type_queryset,
                                   collection_admin_queryset,
                                   view_pemission_queryset)

    def get_collection_type_admin_queryset(self, user):
        return models.Annotation.objects.none()

    def get_collection_admin_queryset(self, user):
        return models.Annotation.objects.none()

    def get_view_permission_queryset(self, user):
        return models.Annotation.objects.none()

    def get_open_queryset(self):
        return models.Annotation.objects.none()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import utils
from irekua_rest_api import serializers


class CollectionTypeLicenceTypeViewSet(mixins.RetrieveModelMixin,
                                       mixins.DestroyModelMixin,
                                       utils.CustomViewSetMixin,
                                       GenericViewSet):
    queryset = models.CollectionType.licence_types.through.objects.all()  # pylint: disable=E1101
    
    serializer_mapping = utils.SerializerMapping.from_module(
        serializers.object_types.data_collections.licences)

    permission_mapping = utils.PermissionMapping()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import mixins
from rest_framework.viewsets import GenericViewSet

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAdmin
from irekua_rest_api.permissions import IsDeveloper
from irekua_rest_api.permissions import IsAuthenticated


class MimeTypeViewSet(mixins.RetrieveModelMixin,
                      mixins.DestroyModelMixin,
                      mixins.UpdateModelMixin,
                      utils.CustomViewSetMixin,
                      GenericViewSet):
    queryset = models.MimeType.objects.all()  # pylint: disable=E1101

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.object_types.mime_types))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.RETRIEVE: IsAuthenticated,
        utils.Actions.DESTROY: IsAdmin,
    }, default=IsDeveloper | IsAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Licence

from irekua_rest_api.serializers.object_types import licences
from irekua_rest_api.serializers.users import users


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = Licence
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Licence
        fields = (
            'url',
            'id',
            'licence_type',
            'created_on',
            'is_active',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    licence_type = licences.SelectSerializer(many=False, read_only=True)
    signed_by = users.SelectSerializer(many=False, read_only=True)

    class Meta:
        model = Licence
        fields = (
            'url',
            'id',
            'licence_type',
            'created_on',
            'document',
            'metadata',
            'signed_by',
            'is_active',
            'collection'
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Licence
        fields = (
            'licence_type',
            'document',
            'metadata',
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        try:
            collection = self.context['collection']
        except KeyError:
            collection = None

        self.collection = collection

        try:
            self.fields['licence_type'].queryset = (
                self.collection.collection_type.licence_types)
        except (KeyError, AttributeError):
            pass

    def create(self, validated_data):
        user = self.context['request'].user
        collection = self.context['collection']

        validated_data['signed_by'] = user
        validated_data['collection'] = collection

        return super().create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Collection

from irekua_rest_api.serializers.users import users
from . import data_collections


MODEL = Collection.administrators.through  # pylint: disable=E1101


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    user = serializers.CharField(
        read_only=True,
        source='user.username')

    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
            'user',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    user = users.SelectSerializer(
        many=False,
        read_only=True)
    collection = data_collections.SelectSerializer(
        many=False,
        read_only=True)

    class Meta:
        model = MODEL
        fields = (
            'url',
            'id',
            'collection',
            'user',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = MODEL
        fields = (
            'user',
        )

    def create(self, validated_data):
        collection = self.context['collection']
        validated_data['collection'] = collection
        return super().create(validated_data)



from irekua_database.models import Synonym
from .utils import BaseFilter


search_fields = (
    'source__value',
    'target__value',
)


class Filter(BaseFilter):
    class Meta:
        model = Synonym
        fields = (
            'source__value',
            'target__value'
        )



from uuid import uuid4

from rest_framework.test import APITestCase

from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import licence_types
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class LicenceTypeTestCase(BaseTestCase, APITestCase):
    serializer = licence_types.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [Users.ADMIN],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN],
        Actions.DESTROY: [
            Users.ADMIN],
    })

    @staticmethod
    def generate_random_json_data():
        data = {
            'name': str(uuid4()),
            'description': 'Random licence type',
            'metadata_schema': simple_JSON_schema(),
            'years_valid_for': 3,
            'document_template': None,
            'can_view': False,
            'can_download': False,
            'can_view_annotations': False,
            'can_annotate': False,
            'can_vote_annotations': False,
        }
        return data



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db.models import Model
from django.shortcuts import get_object_or_404
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response


class Actions(object):
    LIST = 'list'
    UPDATE = 'update'
    PARTIAL_UPDATE = 'partial_update'
    METADATA = 'metadata'
    CREATE = 'create'
    DESTROY = 'destroy'
    RETRIEVE = 'retrieve'

    DEFAULT_ACTIONS = [
        LIST,
        UPDATE,
        PARTIAL_UPDATE,
        METADATA,
        CREATE,
        RETRIEVE,
        DESTROY,
    ]


class AdditionalActionsMixin(object):
    def list_related_object_view(self, queryset=None):
        if queryset is None:
            queryset = self.filter_queryset(self.get_queryset())

        page = self.paginate_queryset(queryset)

        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    def get_related_object(
            self,
            queryset,
            pk_field='pk',
            extra=None):
        serializer = self.get_serializer(data=self.request.data)

        if not serializer.is_valid():
            raise ValidationError(serializer.errors)

        extra_dict = {}
        if extra is not None:
            for extra_value in extra:
                extra_dict[extra_value] = (
                    serializer.validated_data.pop(extra_value))

        _, pk = serializer.validated_data.popitem()

        if isinstance(pk, Model):
            return pk, extra_dict

        query = {pk_field: pk}
        related_object = get_object_or_404(
            queryset,
            **query)

        return related_object, extra_dict

    def create_related_object_view(self, extra=None):
        serializer = self.get_serializer(data=self.request.data)

        if not serializer.is_valid():
            raise ValidationError(serializer.errors)

        extra_dict = {}
        if extra is not None:
            for extra_value in extra:
                extra_dict[extra_value] = (
                    serializer.validated_data.pop(extra_value))

        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    def add_related_object_view(
            self,
            model,
            name,
            pk_field='pk',
            extra=None):
        related_object, extra_dict = self.get_related_object(
            model,
            extra=extra,
            pk_field=pk_field)

        view_object = self.get_object()
        method_name = 'add_{name}'.format(name=name)
        method = getattr(view_object, method_name)
        method(related_object, **extra_dict)
        return Response(status=status.HTTP_200_OK)

    def remove_related_object_view(
            self,
            name,
            pk_field='pk',
            many=True):
        view_object = self.get_object()
        suffix = 's' if many else '_set'
        related_manager_name = '{name}{suffix}'.format(
            name=name,
            suffix=suffix)
        related_objects = getattr(view_object, related_manager_name).all()

        related_object, _ = self.get_related_object(
            related_objects,
            pk_field=pk_field)

        method_name = 'remove_{name}'.format(name=name)
        method = getattr(view_object, method_name)
        method(related_object)
        return Response(status=status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework.viewsets import ModelViewSet
from rest_framework.decorators import action

from irekua_database import models
from irekua_rest_api import serializers
from irekua_rest_api import filters
from irekua_rest_api import utils

from irekua_rest_api.permissions import IsAuthenticated
from irekua_rest_api.permissions import IsAdmin
import irekua_rest_api.permissions.sites as permissions


class SiteViewSet(utils.CustomViewSetMixin, ModelViewSet):
    queryset = models.Site.objects.all()  # pylint: disable=E1101
    filterset_class = filters.sites.Filter
    search_fields = filters.sites.search_fields

    serializer_mapping = (
        utils.SerializerMapping
        .from_module(serializers.sites)
        .extend(
            types=serializers.object_types.sites.ListSerializer,
            add_type=serializers.object_types.sites.CreateSerializer,
        ))

    permission_mapping = utils.PermissionMapping({
        utils.Actions.UPDATE: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsAdmin
            )
        ],
        utils.Actions.DESTROY: [
            IsAuthenticated,
            (
                permissions.IsCreator |
                IsAdmin
            )
        ],
        'add_type': [IsAuthenticated, IsAdmin]
    }, default=IsAuthenticated)

    def get_serializer_class(self):
        if self.action == utils.Actions.RETRIEVE:
            user = self.request.user
            site = self.get_object()

            if site.has_coordinate_permission(user):
                return serializers.sites.FullDetailSerializer
            return serializers.sites.DetailSerializer

        return super().get_serializer_class()

    def get_queryset(self):
        if self.action == 'types':
            return models.SiteType.objects.all()  # pylint: disable=E1101

        return super().get_queryset()

    @action(
        detail=False,
        methods=['GET'],
        filterset_class=filters.site_types.Filter,
        search_fields=filters.site_types.search_fields)
    def types(self, request):
        return self.list_related_object_view()

    @types.mapping.post
    def add_type(self, request):
        return self.create_related_object_view()



from uuid import uuid4
from random import sample

from rest_framework.test import APITestCase

from irekua_database.models import TermType, Term
from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import synonyms
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class SynonymTestCase(BaseTestCase, APITestCase):
    serializer = synonyms.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: [
            Users.ADMIN,
            Users.CURATOR,
        ],
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            Users.CURATOR,
        ],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            Users.CURATOR,
        ],
        Actions.DESTROY: [
            Users.ADMIN,
            Users.CURATOR
        ],
        })

    def setUp(self):
        super(SynonymTestCase, self).setUp()
        term_type = TermType.objects.create(
            name=str(uuid4()),
            description='random term type',
            is_categorical=True,
            metadata_schema=simple_JSON_schema(),
            synonym_metadata_schema=simple_JSON_schema())
        self.terms = [
            Term.objects.create(
                term_type=term_type,
                value=str(uuid4()))
            for _ in range(40)
        ]

    def generate_random_json_data(self):
        term1, term2 = sample(self.terms, 2)
        data = {
            'source': term1.pk,
            'target': term2.pk,
            'metadata': {}
        }
        return data



from uuid import uuid4

from rest_framework.test import APITestCase

from irekua_database.models import SiteType, User
from irekua_database.utils import simple_JSON_schema
from irekua_rest_api.serializers import sites
from .utils import (
    BaseTestCase,
    Users,
    Actions,
    create_permission_mapping_from_lists
)


class SiteTestCase(BaseTestCase, APITestCase):
    serializer = sites.CreateSerializer
    permissions = create_permission_mapping_from_lists({
        Actions.LIST: Users.ALL_AUTHENTICATED_USERS,
        Actions.CREATE: Users.ALL_AUTHENTICATED_USERS,
        Actions.RETRIEVE: Users.ALL_AUTHENTICATED_USERS,
        Actions.UPDATE: [
            Users.ADMIN,
            ],
        Actions.PARTIAL_UPDATE: [
            Users.ADMIN,
            ],
        Actions.DESTROY: [
            Users.ADMIN,
            ],
        })

    def setUp(self):
        super(SiteTestCase, self).setUp()

        self.random_user = User.objects.create(
            username=str(uuid4()),
            password=str(uuid4()))
        self.site_type = SiteType.objects.create(
            name=str(uuid4()),
            description='Random Site Type',
            metadata_schema=simple_JSON_schema())

    def get_serializer_context(self):
        class DummyRequest(object):
            def __init__(self, user):
                self.user = user

        request = DummyRequest(user=self.random_user)
        return {'request': request}

    def generate_random_json_data(self):
        data = {
            'name': str(uuid4()),
            'site_type': self.site_type.name,
            'metadata': {},
            'longitude': 0,
            'latitude': 0,
        }
        return data



from irekua_database.models import CollectionDevice
from .utils import BaseFilter


search_fields = (
    'physical_device__device__model',
    'physical_device__device__brand__name',
    'physical_device__device__device_type__name',
)


class Filter(BaseFilter):
    class Meta:
        model = CollectionDevice
        fields = (
            'physical_device__device__model',
            'physical_device__device__brand__name',
            'physical_device__device__device_type__name',
            'physical_device__bundle',
            'physical_device__created_by__username',
            'physical_device__created_by__first_name',
            'physical_device__created_by__last_name',
            'internal_id',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.auth.models import Permission
from rest_framework import serializers

from irekua_database.models import Role


class SelectPermissionSerializer(serializers.ModelSerializer):
    codename = serializers.SlugRelatedField(
        many=False,
        read_only=False,
        slug_field='codename',
        queryset=Permission.objects.filter(content_type__model='collection'))

    class Meta:
        model = Permission
        fields = (
            'codename',
        )


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = Role
        fields = (
            'url',
            'name',
        )


class ListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Role
        fields = (
            'url',
            'name',
            'description',
            'icon',
        )


class DetailSerializer(serializers.HyperlinkedModelSerializer):
    permissions = serializers.SlugRelatedField(
        many=True,
        read_only=True,
        slug_field='codename')

    class Meta:
        model = Role
        fields = (
            'url',
            'name',
            'description',
            'permissions',
            'icon',
            'modified_on',
            'created_on',
        )


class CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Role
        fields = (
            'name',
            'description',
            'icon',
        )


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Role
        fields = (
            'description',
            'icon',
        )



from irekua_database.models import SamplingEventType
from .utils import BaseFilter


search_fields = (
    'name',
)


class Filter(BaseFilter):
    class Meta:
        model = SamplingEventType
        fields = (
            'name',
            'restrict_device_types',
            'restrict_site_types',
        )



from rest_framework import routers
from irekua_rest_api import views


main_router = routers.DefaultRouter()
main_router.register(
    r'annotations',
    views.AnnotationViewSet)
main_router.register(
    r'collections',
    views.CollectionViewSet)
main_router.register(
    r'devices',
    views.DeviceViewSet)
main_router.register(
    r'items',
    views.ItemViewSet)
main_router.register(
    r'sampling_events',
    views.SamplingEventViewSet)
main_router.register(
    r'sites',
    views.SiteViewSet)
main_router.register(
    r'terms',
    views.TermTypeViewSet)
main_router.register(
    r'users',
    views.UserViewSet)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework import serializers

from irekua_database.models import Annotation
from irekua_database.models import AnnotationTool

from irekua_rest_api.serializers.terms.terms import ListSerializer as TermListSerializer
from irekua_rest_api.serializers.terms.terms import ComplexTermSerializer
from irekua_rest_api.serializers.users.users import ListSerializer as UserListSerializer


class SelectSerializer(serializers.ModelSerializer):
    class Meta:
        model = Annotation
        fields = (
            'url',
            'id',
        )


class ListSerializer(serializers.ModelSerializer):
    labels = TermListSerializer(many=True)
    created_by = UserListSerializer()

    class Meta:
        model = Annotation
        fields = (
            'url',
            'id',
            'item',
            'event_type',
            'annotation',
            'annotation_type',
            'created_on',
            'created_by',
            'labels'
        )


class DetailSerializer(serializers.ModelSerializer):
    labels = ComplexTermSerializer(many=True)

    class Meta:
        model = Annotation
        fields = (
            'url',
            'id',
            'annotation_tool',
            'item',
            'event_type',
            'labels',
            'annotation_type',
            'annotation',
            'annotation_configuration',
            'certainty',
            'quality',
            'commentaries',
            'created_on',
            'modified_on',
            'created_by',
            'modified_by',
        )


class AnnotationToolSerializer(serializers.Serializer):
    name = serializers.CharField(required=True)
    version = serializers.CharField(required=True)
    description = serializers.CharField(required=False)
    logo = serializers.CharField(required=False)
    website = serializers.CharField(required=False)
    configuration_schema = serializers.JSONField(required=False)


class CreateSerializer(serializers.ModelSerializer):
    annotation_tool = AnnotationToolSerializer()

    class Meta:
        model = Annotation
        fields = (
            'event_type',
            'annotation',
            'labels',
            'certainty',
            'quality',
            'commentaries',
            'annotation_tool',
            'annotation_configuration',
            'annotation_type',
        )

    def create(self, validated_data):
        item = self.context['item']
        user = self.context['request'].user

        annotation_tool_info = validated_data.pop('annotation_tool')

        name = annotation_tool_info.pop('name')
        version = annotation_tool_info.pop('version')
        annotation_tool, _ = AnnotationTool.objects.get_or_create(
            name=name,
            version=version,
            defaults=annotation_tool_info)
        annotation_tool.save()

        validated_data['item'] = item
        validated_data['created_by'] = user
        validated_data['modified_by'] = user
        validated_data['annotation_tool'] = annotation_tool

        labels = validated_data.pop('labels')
        annotation = Annotation(**validated_data)
        annotation.save()

        annotation.labels.set(labels)
        annotation.save()
        return annotation


class UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = Annotation
        fields = (
            'annotation',
            'labels',
            'certainty',
            'quality',
            'commentaries',
            'annotation_configuration',
        )

    def update(self, instance, validated_data):
        user = self.context['request'].user

        validated_data['modified_by'] = user
        return super().update(instance, validated_data)
